<?php
/**
 * Plugin Name: WooCommerce Order Product Count
 * Plugin URI: https://dkjensen.com
 * Description: Export multiple orders product counts to a PDF
 * Version: 1.3.1
 * Author: David Jensen
 * Author URI: http://dkjensen.com
 * License: GPL2
 */


/**
 * Includes WooCommerce functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) || ! function_exists( 'is_woocommerce_active' ) )
	require_once 'woocommerce/woo-functions.php';

/**
 * Check if WooCommerce is active
 */
if ( ! is_woocommerce_active() ) {
	add_action( 'admin_notices', 'WC_Order_Product_Count::woocommerce_inactive_notice' );
	return;
}

define( 'WOO_EXPORT_ORDERS', plugin_dir_path( __FILE__ ) );


class WC_Order_Product_Count {

	public function __construct() {
		$this->hooks();
		$this->includes();
	}


	/**
	 * Look for request to export orders
	 * 
	 * @since 1.3
	 * @return void
	 */
	public function init() {
		if( isset( $_REQUEST['weo_export'] ) && current_user_can( 'edit_shop_orders' ) ) {
			$this->export();
		}
	}

	/**
	 * Load our includes
	 * 
	 * @since 1.3
	 * @return void
	 */
	public function includes() {
		require_once 'includes/wc-order-product-count-functions.php';
		require_once 'includes/class-wc-order-product-count-pdf.php';
	}


	/**
	 * Hooks
	 * 
	 * @since 1.3
	 * @return void
	 */
	public function hooks() {
		add_action( 'admin_footer',  array( $this, 'woocommerce_export_orders_menu' ), 10 );
		add_action( 'load-edit.php', array( $this, 'woocommerce_export_orders_bulk_action' ) );

		add_action( 'init', array( $this, 'init' ) );
	}


	/**
	 * Start the generation of the PDF
	 * 
	 * @since 1.3
	 * @return type
	 */
	public function export() {
		WC_OPCPDF()->generate();
	}


	/**
	 * Admin error notice to ensure WooCommerce is active
	 * 
	 * @since 1.0
	 * @return string
	 */
	public static function woocommerce_inactive_notice() {
		if ( current_user_can( 'activate_plugins' ) ) {
			$output  = '<div id="message" class="error"><p>';
			$output .= sprintf( __( 'WooCommerce Export Orders is inactive. WooCommerce must be active in order to use WooCommerce Export Orders.', 'export-orders' ) );
			$output .= '</p></div>';
			print $output;
		}
	}


	/**
	 * Add the Orders Product Count menu item to bulk actions
	 * 
	 * @since 1.0
	 * @return string
	 */
	public function woocommerce_export_orders_menu() {
		global $post_type;
		if ( 'shop_order' == $post_type ) {
			?>
			<script type="text/javascript">
			jQuery(document).ready(function() {
				jQuery('<option>').val('export_orders').text('<?php _e( 'Orders Product Count', 'woocommerce-export' )?>').appendTo("select[name='action']");
				jQuery('<option>').val('export_orders').text('<?php _e( 'Orders Product Count', 'woocommerce-export' )?>').appendTo("select[name='action2']");
			});
			</script>
			<?php
		}
	}


	/**
	 * Are we exporting the order count from the bulk actions menu?
	 * 
	 * @since 1.0
	 * @return void
	 */
	public function woocommerce_export_orders_bulk_action() {
		if( ! isset( $_REQUEST['action'] ) || empty( $_REQUEST['action'] ) )
			return;
		$action = $_REQUEST['action'];
		if ( $action == 'export_orders' ) {
			// If no posts selected do nothing
			if( empty( $_REQUEST['post'] ) )
				return; 
			// If posts are selected join them into string
			if( is_array( $_REQUEST['post'] ) && ! empty( $_REQUEST['post'] ) ) {
				$posts = implode( ',', $_REQUEST['post'] );
			}
			$forward = wp_nonce_url( admin_url( 'admin.php' ), 'export-orders' );
			$forward = add_query_arg( array( 'weo_export' => 'woocommerce-export-orders', 'posts' => $posts ), $forward );
			wp_redirect( $forward );
			exit();
		}
		return;
	}
}
new WC_Order_Product_Count;