<?php

class WC_Order_Product_Count_PDF {
	/**
	 * The single instance of the class.
	 */
	protected static $_instance = null;

	/**
	 * Store our TCPDF object
	 * 
	 * @since 1.3
	 */
	protected $pdf;

	/**
	 * Store our orders
	 * 
	 * @since 1.3 
	 */
	public $orders = array();

	/**
	 * Store our table columns
	 * 
	 * @since 1.3
	 */
	public $columns = array();


	public function __construct() {
		$this->includes();
		$this->setup_vars();
	}


	/**
	 * Only allow one instance of this class to be instantiated
	 * 
	 * @since 1.3
	 * @return type
	 */
	public static function instance() {
		if( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}


	/**
	 * Generate the PDF for export
	 * 
	 * @since 1.3
	 * @return type
	 */
	public function generate() {
		// Instantiate a new TCPDF object
		$this->pdf = new WC_Export_TCPDF( PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, get_bloginfo( 'charset' ), false );
		$this->pdf->SetCreator( PDF_CREATOR );
		$this->pdf->SetAuthor( get_bloginfo( 'name' ) );
		$this->pdf->SetTitle( apply_filters( 'wc_order_product_count_pdf_title', __( 'Export Orders', 'wcopcpdf' ) ) );
		$this->pdf->SetMargins( PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT );
		$this->pdf->SetHeaderMargin( PDF_MARGIN_HEADER );
		$this->pdf->SetFooterMargin( PDF_MARGIN_FOOTER );
		$this->pdf->SetAutoPageBreak( TRUE, PDF_MARGIN_BOTTOM );
		$this->pdf->setImageScale( PDF_IMAGE_SCALE_RATIO );
		$this->pdf->AddPage();

		$totals = $this->get_order_product_count( $this->get_orders() );

		if( $totals < 1 ) {
			wp_die( __( 'Empty order product count.', 'export-orders' ) );
		}

		// Include our stylesheet to get things started
		$output  = sprintf( '<style>%s</style>', file_get_contents( WOO_EXPORT_ORDERS . 'css/woocommerce-pdf.css' ) );

		// Add column headers
		$output .= "<table><tr>";
		foreach( $this->get_columns() as $key => $column ) {
			$label   = ! empty( $column['label'] ) ? esc_attr( $column['label'] ) : '&nbsp;';
			$output .= sprintf( '<td class="table-head product-%s" style="width: %s;">%s</td>', $key, $column['size'], $label );
		}
		$output .= "</tr>";

		$i = 0;

		// Begin displaying our product quantity rows
		foreach( $totals as $product => $quantity ) { $i++;
			$fill = null;
				
			if( $i % 2 === 0 ) {
				$fill = 'fill';
				$i = 0;
			}

			$output .= $this->print_row( $product, $quantity, $fill );
		}

		$output .= "</table>";

		// Add our product rows
		$this->pdf->writeHTMLCell( 0, 0, '', '', $output, 0, 1 );

		// Output the PDF
		$this->pdf->Output( 'woocommerce-export-order-count-' . time() . '.pdf', 'D' );

		// Do not display another other output
		exit;
	}


	/**
	 * Include TCPDF
	 * 
	 * @since 1.3
	 * @return void
	 */
	public function includes() {
		require_once WOO_EXPORT_ORDERS . 'lib/tcpdf/config/tcpdf_config.php';
		require_once WOO_EXPORT_ORDERS . 'lib/tcpdf/tcpdf.php';
		require_once WOO_EXPORT_ORDERS . 'lib/tcpdf/woo_export_tcpdf.php';
	}


	/**
	 * Setup some basic variables for the output of the PDF
	 * 
	 * @since 1.3
	 * @return void
	 */
	public function setup_vars() {
		$orders = ! empty( $_REQUEST['posts'] ) ? explode( ',', $_REQUEST['posts'] ) : array();

		$this->orders  = array_map( 'absint', array_filter( $orders ) );
		$this->columns = array(
			'thumbnail' => array(
				'label' => null,
				'size'  => '15%',
				'callback' => 'wcopcpdf_column_thumbnail'
			),
			'id' => array(
				'label' => __( 'ID', '@' ),
				'size'  => '10%',
				'callback' => 'wcopcpdf_column_id'
			),
			'product' => array(
				'label' => __( 'Product', 'wcopcpdf' ),
				'size'  => '55%',
				'callback' => 'wcopcpdf_column_product'
			),
			'quantity' => array(
				'label' => __( 'Quantity', 'wcopcpdf' ),
				'size'  => '20%',
				'callback' => 'wcopcpdf_column_quantity'
			),
		);
	}


	/**
	 * Get the orders array to output
	 * 
	 * @since 1.3
	 * @return array
	 */
	public function get_orders() {
		return apply_filters( 'wc_order_product_count_orders', apply_filters( 'wc_order_product_count_orders', $this->orders ) );
	}


	/**
	 * Returns the array of columns to display
	 * 
	 * @since 1.3
	 * @return array
	 */
	public function get_columns() {
		return apply_filters( 'wc_order_product_count_columns', $this->columns );
	}


	/**
	 * Retrieves the sum of items from a list of orders
	 * 
	 * @param array $orders 
	 * @return array
	 */
	public function get_order_product_count( $orders ) {
		if( ! isset( $orders ) || empty( $orders ) || ! is_array( $orders ) )
            return 0;
            
		$total = array();
		foreach( $orders as $order ) {
			$order = new WC_Order( $order );
			$items = $order->get_items();
			foreach( $items as $item ) {
                $product_id = ! empty( $item['variation_id'] ) ? $item['variation_id'] : $item['product_id'];

                $total[$product_id] = isset( $total[$product_id] ) ? $total[$product_id] + $item['qty'] : $item['qty'];
			}
        }
        
		return $total;
	}


	/**
	 * Generate the output of each product quantity row
	 * 
	 * @param int $product_id 
	 * @param int $quantity 
	 * @param string $fill
	 * @since 1.3
	 * @return string 
	 */
	public function print_row( $product_id, $quantity, $fill ) {
		$output = '';

		$output .= sprintf( '<tr><td colspan="%s" class="%s"><table style="padding: 5px;"><tr>', sizeof( $this->get_columns() ), $fill );

		foreach( $this->get_columns() as $key => $column ) {
			$output .= sprintf( '<td class="table-cell product-%s %s" style="width: %s;">', $key, $fill, $column['size'] );
			$output .= call_user_func( $column['callback'], $product_id, $quantity, $this->get_orders() );
			$output .= '</td>';
		}

		$output .= '</tr></table></td></tr>';

		return $output;
	}

}

function WC_OPCPDF() {
	return WC_Order_Product_Count_PDF::instance();
}
